/*
 * Copyright (C) 2001  Patrick Ekman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: mod_vhost_cdb.c,v 1.7 2001/07/01 22:23:45 pat Exp $
 */

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "http_request.h"

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <cdb.h>

typedef struct {
    char *cdb;
} vcdb_cfg;

module MODULE_VAR_EXPORT vhost_cdb_module;


static void
*vhost_cdb_create_server_config(pool *p, server_rec *s)
{
    vcdb_cfg *cfg;

    cfg = ap_pcalloc(p, sizeof(vcdb_cfg));
    return cfg;
}


static const char
*cmd_vhostcdb(cmd_parms *cmd, void *dummy, char *val)
{
    vcdb_cfg *cfg = ap_get_module_config(cmd->server->module_config,
					 &vhost_cdb_module);
    cfg->cdb = ap_pstrdup(cmd->pool, val);
    return NULL;
}


static int
vhost_cdb_translate(request_rec *r)
{
    vcdb_cfg *cfg = ap_get_module_config(r->server->module_config,
					 &vhost_cdb_module);
    int fd, result;
    struct cdb c;
    char *uri, *path;
    char *hostname, *chr;
    int rval = DECLINED;

    if (cfg->cdb == NULL) {
	return DECLINED;
    }

    uri = r->uri;
    hostname = ap_get_server_name(r);
    chr = hostname;
    while (*chr != '\0') *chr++ = tolower(*chr);

    if (uri[0] != '/') {
	return DECLINED;
    }
    if ( (fd = ap_popenf(r->pool, cfg->cdb, O_RDONLY, 0)) == -1) {
	return DECLINED;
    }

    bzero(&c, sizeof(c));
    cdb_init(&c, fd);
    result = cdb_find(&c, hostname, strlen(hostname));
    if (result == -1) {
	perror("cdb_find");
	rval = HTTP_INTERNAL_SERVER_ERROR;
	goto cleanup;
    }
    else if (result == 0) {
	rval = DECLINED;
	goto cleanup;
    }

    path = ap_palloc(r->pool, cdb_datalen(&c) + 1);
    if (cdb_read(&c, path, cdb_datalen(&c), cdb_datapos(&c)) == -1) {
	perror("cdb_read");
	rval = HTTP_INTERNAL_SERVER_ERROR;
	goto cleanup;
    }
    path[cdb_datalen(&c)] = 0;

    r->filename = ap_pstrcat(r->pool, path, uri, NULL);

    if (strncmp(uri + strspn(uri, "/"), "cgi-bin/", 8) == 0) {
	r->handler = "cgi-script";
	ap_table_setn(r->notes, "alias-forced-type", r->handler);
    }

    rval = OK;

cleanup:
    cdb_free(&c);
    return rval;
}



static const command_rec vhost_cdb_cmds[] =
{
    {
	"VhostCDB",
	cmd_vhostcdb,
	NULL,
	RSRC_CONF,
	TAKE1,
	"Virtual host database file location."
    },
    { NULL }
};
	


module MODULE_VAR_EXPORT vhost_cdb_module =
{
    STANDARD_MODULE_STUFF,
    NULL,			/* module initializer */
    NULL,			/* per-directory config creator */
    NULL,			/* dir config merger */
    vhost_cdb_create_server_config,	/* server config creator */
    NULL,			/* server config merger */
    vhost_cdb_cmds,		/* command table */
    NULL,			/* [9] list of handlers */
    vhost_cdb_translate,	/* [2] filename-to-URI translation */
    NULL,			/* [5] check/validate user_id */
    NULL,			/* [6] check user_id is valid *here* */
    NULL,			/* [4] check access by host address */
    NULL,			/* [7] MIME type checker/setter */
    NULL,			/* [8] fixups */
    NULL,			/* [10] logger */
    NULL,			/* [3] header parser */
    NULL,			/* process initializer */
    NULL,			/* process exit/cleanup */
    NULL                        /* [1] post read_request handling */
};

