/*
 * Copyright (C) 2001  Patrick Ekman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: vhostmake.c,v 1.2 2001/06/30 00:26:09 pat Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <cdb_make.h>

#define LINE_MAX 512

static int parse_line(char *, char **, char **);
static void usage(char *);

int
main(int argc, char **argv)
{
    int fd;
    struct cdb_make cdb;
    char line[LINE_MAX];
    char *key, *value;
    char result;
    unsigned lno = 0;

    if (argc != 3) {
	usage(argv[0]);
	exit(1);
    }

    if ( (fd = open(argv[2], O_WRONLY | O_CREAT | O_TRUNC, 0644)) < 0) {
	perror(argv[2]);
	exit(errno);
    }
    if (cdb_make_start(&cdb, fd) < 0) {
	perror(argv[2]);
	exit(errno);
    }

    while (fgets(line, LINE_MAX, stdin) != NULL) {
	lno++;
	result = parse_line(line, &key, &value);
	if (result == 0) continue;
	if (result == -1) {
	    fprintf(stderr, "Line %d: syntax error.\n", lno);
	    exit(1);
	}
	if (result != 1) {
	    fprintf(stderr, "Yikes.  Reached the unreachable.\n");
	    exit(1);
	}

	if (cdb_make_add(&cdb, key, strlen(key), value, strlen(value)) < 0) {
	    perror(argv[2]);
	    exit(errno);
	}
    }

    if (cdb_make_finish(&cdb) < 0) {
	perror(argv[2]);
	exit(errno);
    }
    if (fsync(fd) < 0) {
	perror(argv[2]);
	exit(errno);
    }
    if (close(fd) < 0) {
	perror(argv[2]);
	exit(errno);
    }
    if (rename(argv[2], argv[1]) < 0) {
	perror(argv[1]);
	exit(errno);
    }

    return 0;
}


void
usage(char *progname)
{
    fprintf(stderr, "Usage: %s dbname tmpfile\n", progname);
}


/*
 * Parses the contents of line (which are partially mangled in the
 * process).  Returns 1 if the line is valid and results are stored in
 * domain and path.  Returns 0 if the line is empty or contains a
 * comment.  Returns -1 on a syntax error.
 */
int parse_line(char *line, char **domain, char **path)
{
    char *c = line;
    char *dom, *pth;
    char *startspace;
    int len = strlen(line);

    /* Trim any trailing newlines */
    if (len > 0 && (line[len-1] == '\n' || line[len-1] == '\r')) len--;
    if (len > 0 && (line[len-1] == '\n' || line[len-1] == '\r')) len--;
    line[len] = '\0';

    /* Check for blank line &/| comment */
    if (*(line + strspn(line, " \t")) == '#') return 0;
    if (strspn(line, " \t") == len) return 0;

    /* Domain name */
    dom = line;
    while ((*c >= 'a' && *c <= 'z') ||
	   (*c >= 'A' && *c <= 'Z') ||
	   (*c >= '0' && *c <= '9') ||
	   *c == '.' || *c == '-')
	*c++ = tolower(*c);
    if (*c != ':') return -1;
    *c++ = '\0';

    /* Path */
    pth = c;
    if (isspace(*c)) return -1;
    while (*c != '\0') {
	if (isspace(*c) && startspace == NULL) {
	    startspace = c;
	}
	else if (! isspace(*c)) {
	    startspace = NULL;
	}
	c++;
    }

    if (startspace != NULL) *startspace = '\0';
    if (c - pth == 0) return -1;
    *c = '\0';

    /* Return results */
    *domain = dom;
    *path = pth;

    return 1;
}

